from lib import BaseTest


class PublishSourceAdd1Test(BaseTest):
    """
    publish source add: add single source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source add -component=test maverick snap2"
    gold_processor = BaseTest.expand_environ


class PublishSourceAdd2Test(BaseTest):
    """
    publish source add: add multiple sources
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly snapshot create snap3 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source add -component=test,other-test maverick snap2 snap3"
    gold_processor = BaseTest.expand_environ


class PublishSourceAdd3Test(BaseTest):
    """
    publish source add: (re-)add already added source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source add -component=main maverick snap2"
    expectedCode = 1
    gold_processor = BaseTest.expand_environ


class PublishSourceList1Test(BaseTest):
    """
    publish source list: show source changes
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
        "aptly publish source add -component=test maverick snap2",
    ]
    runCmd = "aptly publish source list maverick"
    gold_processor = BaseTest.expand_environ


class PublishSourceList2Test(BaseTest):
    """
    publish source list: show source changes as JSON
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
        "aptly publish source add -component=test maverick snap2",
    ]
    runCmd = "aptly publish source list -json maverick"
    gold_processor = BaseTest.expand_environ


class PublishSourceList3Test(BaseTest):
    """
    publish source list: show source changes (empty)
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source list maverick"
    expectedCode = 1
    gold_processor = BaseTest.expand_environ


class PublishSourceDrop1Test(BaseTest):
    """
    publish source drop: drop source changes
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source drop maverick"
    gold_processor = BaseTest.expand_environ


class PublishSourceUpdate1Test(BaseTest):
    """
    publish source update: Update single source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source update -component=main maverick snap2"
    gold_processor = BaseTest.expand_environ


class PublishSourceUpdate2Test(BaseTest):
    """
    publish source update: Update multiple sources
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly snapshot create snap3 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main,test snap1 snap2",
    ]
    runCmd = "aptly publish source update -component=main,test maverick snap2 snap3"
    gold_processor = BaseTest.expand_environ


class PublishSourceUpdate3Test(BaseTest):
    """
    publish source update: Update not existing source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source update -component=not-existent maverick snap1"
    gold_processor = BaseTest.expand_environ


class PublishSourceReplace1Test(BaseTest):
    """
    publish source replace: Replace existing sources
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly snapshot create snap3 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main,test snap1 snap2",
    ]
    runCmd = "aptly publish source replace -component=main-new,test-new maverick snap2 snap3"
    gold_processor = BaseTest.expand_environ


class PublishSourceRemove1Test(BaseTest):
    """
    publish source remove: Remove single source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main,test snap1 snap2",
    ]
    runCmd = "aptly publish source remove -component=test maverick"
    gold_processor = BaseTest.expand_environ


class PublishSourceRemove2Test(BaseTest):
    """
    publish source remove: Remove multiple sources
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly snapshot create snap3 empty",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main,test,other-test snap1 snap2 snap3",
    ]
    runCmd = "aptly publish source remove -component=test,other-test maverick"
    gold_processor = BaseTest.expand_environ


class PublishSourceRemove3Test(BaseTest):
    """
    publish source remove: Remove not-existing source
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -component=main snap1",
    ]
    runCmd = "aptly publish source remove -component=not-existent maverick"
    expectedCode = 1
    gold_processor = BaseTest.expand_environ



import os
import inspect
from lib import BaseTest


def changesRemove(_, s):
    return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), "")


class EditRepo1Test(BaseTest):
    """
    edit repo: change comment
    """
    fixtureCmds = [
        "aptly repo create repo1",
    ]
    runCmd = "aptly repo edit -comment=Lala repo1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo1", "repo-show")


class EditRepo2Test(BaseTest):
    """
    edit repo: change distribution & component
    """
    fixtureCmds = [
        "aptly repo create -comment=Lala -component=non-free repo2",
    ]
    runCmd = "aptly repo edit -distribution=wheezy -component=contrib repo2"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo2", "repo-show")


class EditRepo3Test(BaseTest):
    """
    edit repo: no such repo
    """
    runCmd = "aptly repo edit repo3"
    expectedCode = 1


class EditRepo4Test(BaseTest):
    """
    edit repo: add uploaders.json
    """
    fixtureCmds = [
        "aptly repo create repo4",
    ]
    runCmd = "aptly repo edit -uploaders-file=${changes}/uploaders2.json repo4"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo4", "repo_show")


class EditRepo5Test(BaseTest):
    """
    edit repo: with broken uploaders.json
    """
    fixtureCmds = [
        "aptly repo create repo5",
    ]
    runCmd = "aptly repo edit -uploaders-file=${changes}/uploaders3.json repo5"
    expectedCode = 1


class EditRepo6Test(BaseTest):
    """
    edit local repo: with missing uploaders.json
    """
    fixtureCmds = [
        "aptly repo create repo6",
    ]
    runCmd = "aptly repo edit -uploaders-file=${changes}/uploaders-not-found.json repo6"
    expectedCode = 1
    outputMatchPrepare = changesRemove


class EditRepo7Test(BaseTest):
    """
    edit local repo: remove uploaders.json
    """
    fixtureCmds = [
        "aptly repo create -uploaders-file=${changes}/uploaders2.json repo7",
    ]
    runCmd = "aptly repo edit -uploaders-file= repo7"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo7", "repo_show")



import requests_unixsocket
import urllib.error
import urllib.parse
import urllib.request
import os
import os.path

from lib import BaseTest


class SystemdAPIHandoverTest(BaseTest):
    aptly_server = None
    socket_path = "/tmp/_aptly_systemdapihandovertest.sock"

    def prepare(self):
        # On Debian they use /lib on other systems /usr/lib.
        systemd_activate = "/usr/lib/systemd/systemd-activate"
        if not os.path.exists(systemd_activate):
            systemd_activate = "/lib/systemd/systemd-activate"
        if not os.path.exists(systemd_activate):
            print("Could not find systemd-activate")
            return
        self.aptly_server = self._start_process("%s -l %s aptly api serve -no-lock" %
                                                (systemd_activate, self.socket_path),)
        super(SystemdAPIHandoverTest, self).prepare()

    def shutdown(self):
        if self.aptly_server is not None:
            self.aptly_server.terminate()
            self.aptly_server.wait()
            self.aptly_server = None
        if os.path.exists(self.socket_path):
            os.remove(self.socket_path)
        super(SystemdAPIHandoverTest, self).shutdown()

    def run(self):
        pass

    """
    Verify we can listen on a unix domain socket.
    """
    def check(self):
        if self.aptly_server is None:
            print("Skipping test as we failed to setup a listener.")
            return
        session = requests_unixsocket.Session()
        r = session.get('http+unix://%s/api/version' % urllib.parse.quote(self.socket_path, safe=''))
        self.check_equal(r.json(), {'Version': os.environ['APTLY_VERSION']})



from lib import BaseTest
import re


class CreateSnapshot1Test(BaseTest):
    """
    create snapshot: from mirror
    """
    fixtureDB = True
    runCmd = "aptly snapshot create snap1 from mirror wheezy-main"

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap1", "snapshot_show", match_prepare=remove_created_at)


class CreateSnapshot2Test(BaseTest):
    """
    create snapshot: no mirror
    """
    fixtureDB = True
    runCmd = "aptly snapshot create snap1 from mirror no-such-mirror"
    expectedCode = 1


class CreateSnapshot3Test(BaseTest):
    """
    create snapshot: duplicate name
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create snap1 from mirror wheezy-main"]
    runCmd = "aptly snapshot create snap1 from mirror wheezy-contrib"
    expectedCode = 1


class CreateSnapshot4Test(BaseTest):
    """
    create snapshot: empty
    """
    runCmd = "aptly snapshot create snap4 empty"

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap4", "snapshot_show", match_prepare=remove_created_at)


class CreateSnapshot5Test(BaseTest):
    """
    create snapshot: empty duplicate name
    """
    fixtureCmds = ["aptly snapshot create snap5 empty"]
    runCmd = "aptly snapshot create snap5 empty"
    expectedCode = 1


class CreateSnapshot6Test(BaseTest):
    """
    create snapshot: from repo
    """
    fixtureCmds = [
        "aptly repo create local-repo",
        "aptly repo add local-repo ${files}"
    ]
    runCmd = "aptly snapshot create snap6 from repo local-repo"

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap6", "snapshot_show", match_prepare=remove_created_at)


class CreateSnapshot7Test(BaseTest):
    """
    create snapshot: no repo
    """
    runCmd = "aptly snapshot create snap1 from repo no-such-repo"
    expectedCode = 1


class CreateSnapshot8Test(BaseTest):
    """
    create snapshot: duplicate name from repo
    """
    fixtureCmds = [
        "aptly repo create local-repo",
        "aptly repo add local-repo ${files}",
        "aptly snapshot create snap8 from repo local-repo"
    ]
    runCmd = "aptly snapshot create snap8 from repo local-repo"
    expectedCode = 1


class CreateSnapshot9Test(BaseTest):
    """
    create snapshot: from empty repo
    """
    fixtureCmds = [
        "aptly repo create local-repo",
    ]
    runCmd = "aptly snapshot create snap9 from repo local-repo"


class CreateSnapshot10Test(BaseTest):
    """
    create snapshot: from empty mirror
    """
    fixtureCmds = [
        "aptly mirror create -ignore-signatures non-free http://repo.aptly.info/system-tests/security.debian.org/debian-security bookworm-security updates/non-free",
        "aptly mirror update -ignore-signatures non-free"
    ]
    runCmd = "aptly snapshot create oh-snap from mirror non-free"



from api_lib import APITest
import os


class VersionAPITest(APITest):
    """
    GET /version
    """

    def check(self):
        self.check_equal(self.get("/api/version").json(), {'Version': os.environ['APTLY_VERSION']})



from lib import BaseTest


class RemoveRepo1Test(BaseTest):
    """
    remove from local repo: as dep
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze local-repo",
        "aptly repo add local-repo ${files}"
    ]
    runCmd = "aptly repo remove local-repo pyspi some"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages local-repo", "repo_show")


class RemoveRepo2Test(BaseTest):
    """
    remove from local repo: as dep with version, key
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze local-repo",
        "aptly repo add local-repo ${files}"
    ]
    runCmd = "aptly repo remove local-repo 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages local-repo", "repo_show")


class RemoveRepo3Test(BaseTest):
    """
    remove from local repo: no such repo
    """
    runCmd = "aptly repo remove local-repo 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"
    expectedCode = 1


class RemoveRepo4Test(BaseTest):
    """
    remove from local repo: dry run
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze local-repo",
        "aptly repo add local-repo ${files}"
    ]
    runCmd = "aptly repo remove -dry-run local-repo 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages local-repo", "repo_show")



from lib import BaseTest


class SearchRepo1Test(BaseTest):
    """
    search repo: regular search
    """
    fixtureDB = True
    fixtureCmds = ["aptly repo create wheezy-main", "aptly repo import wheezy-main wheezy-main Name"]
    runCmd = "aptly repo search wheezy-main '$$Architecture (i386), Name (% *-dev)'"


class SearchRepo2Test(BaseTest):
    """
    search repo: missing repo
    """
    runCmd = "aptly repo search repo-xx 'Name'"
    expectedCode = 1


class SearchRepo3Test(BaseTest):
    """
    search repo: wrong expression
    """
    fixtureDB = True
    fixtureCmds = ["aptly repo create wheezy-main", "aptly repo import wheezy-main wheezy-main Name"]
    expectedCode = 1
    runCmd = "aptly repo search wheezy-main '$$Architecture (i386'"


class SearchRepo4Test(BaseTest):
    """
    search repo: with-deps search
    """
    fixtureDB = True
    fixtureCmds = ["aptly repo create wheezy-main", "aptly repo import wheezy-main wheezy-main Name"]
    runCmd = "aptly repo search -with-deps wheezy-main 'Name (nginx)'"


class SearchRepo5Test(BaseTest):
    """
    search repo: with -format
    """
    fixtureDB = True
    fixtureCmds = ["aptly repo create wheezy-main", "aptly repo import wheezy-main wheezy-main Name"]
    runCmd = "aptly repo search -format='{{.Package}}#{{.Version}}' wheezy-main '$$Architecture (i386), Name (% *-dev)'"


class SearchRepo6Test(BaseTest):
    """
    search repo: without query
    """
    fixtureDB = True
    fixtureCmds = ["aptly repo create wheezy-main", "aptly repo import wheezy-main wheezy-main Name"]
    runCmd = "aptly repo search wheezy-main"



import tempfile


class TestOut:
    def __init__(self):
        self.tmp_file = tempfile.NamedTemporaryFile(delete=False)
        self.read_pos = 0

    def fileno(self):
        return self.tmp_file.fileno()

    def write(self, text):
        self.tmp_file.write(text.encode())

    def get_contents(self):
        self.tmp_file.seek(self.read_pos, 0)
        return self.tmp_file.read().decode("utf-8")

    def close(self):
        self.tmp_file.close()

    def clear(self):
        self.read_pos = self.tmp_file.tell()



from lib import BaseTest


class CopyRepo1Test(BaseTest):
    """
    copy in local repo: simple copy
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo copy repo1 repo2 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo1_show")
        self.check_cmd_output("aptly repo show -with-packages repo2", "repo2_show")


class CopyRepo2Test(BaseTest):
    """
    copy in local repo: simple copy w/deps
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly -architectures=i386,amd64 repo copy -with-deps repo1 repo2 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo1_show")
        self.check_cmd_output("aptly repo show -with-packages repo2", "repo2_show")


class CopyRepo3Test(BaseTest):
    """
    copy in local repo: simple copy w/deps but w/o archs
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo copy -with-deps repo1 repo2 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"
    expectedCode = 1


class CopyRepo4Test(BaseTest):
    """
    copy in local repo: dry run
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo copy -dry-run repo1 repo2 'pyspi (>> 0.6.1-1.3)' libboost-program-options-dev_1.49.0.1_i386"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo1_show")
        self.check_cmd_output("aptly repo show -with-packages repo2", "repo2_show")


class CopyRepo5Test(BaseTest):
    """
    copy in local repo: wrong dep
    """
    sortOutput = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo copy repo1 repo2 'pyspi >> 0.6.1-1.3)'"
    expectedCode = 1


class CopyRepo6Test(BaseTest):
    """
    copy in local repo: same src and dest
    """
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo copy repo1 repo1 pyspi"
    expectedCode = 1


class CopyRepo7Test(BaseTest):
    """
    copy in local repo: no dst
    """
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo copy repo1 repo2 pyspi"
    expectedCode = 1


class CopyRepo8Test(BaseTest):
    """
    copy in local repo: no src
    """
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo2",
    ]
    runCmd = "aptly repo copy repo1 repo2 pyspi"
    expectedCode = 1



from lib import BaseTest


class ImportRepo1Test(BaseTest):
    """
    import to local repo: simple import
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo import wheezy-main repo1 'nginx (>= 1.2)' unpaper_0.4.2-1_amd64"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo_show")


class ImportRepo2Test(BaseTest):
    """
    import to local repo: import w/deps
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly -architectures=i386,amd64 repo import -with-deps wheezy-main repo1 dpkg_1.16.12_i386 userinfo_2.2-3_amd64"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo_show")


class ImportRepo3Test(BaseTest):
    """
    import to local repo: simple move w/deps but w/o archs
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo import -with-deps wheezy-contrib repo1 redeclipse"
    expectedCode = 1


class ImportRepo4Test(BaseTest):
    """
    import to local repo: dry run
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly -architectures=i386,amd64 repo import -dry-run -with-deps wheezy-contrib repo1 redeclipse-dbg"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo_show")


class ImportRepo5Test(BaseTest):
    """
    import to local repo: wrong dep
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo import wheezy-contrib repo1 'pyspi >> 0.6.1-1.3)'"
    expectedCode = 1


class ImportRepo6Test(BaseTest):
    """
    import to local repo: non-updated mirror
    """
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
        "aptly mirror create --ignore-signatures mirror1 http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian/ stretch",
    ]
    runCmd = "aptly repo import mirror1 repo1 nginx"
    expectedCode = 1


class ImportRepo7Test(BaseTest):
    """
    import to local repo: no dst
    """
    fixtureDB = True
    fixtureCmds = [
    ]
    runCmd = "aptly repo import wheezy-contrib repo1 nginx"
    expectedCode = 1


class ImportRepo8Test(BaseTest):
    """
    import to local repo: no src
    """
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo import wheezy-main repo1 pyspi"
    expectedCode = 1


class ImportRepo9Test(BaseTest):
    """
    import to local repo: import with complex query
    """
    sortOutput = True
    fixtureDB = True
    fixtureCmds = [
        "aptly repo create -comment=Cool -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo import wheezy-main repo1 '(httpd, $$Source (=nginx)) | exim4'"



"""
Test library.
"""

import difflib
import http.server
import inspect
import json
import os
import posixpath
import pprint
import re
import shlex
import shutil
import socketserver
import string
import subprocess
import threading
import urllib.error
import urllib.parse
import urllib.request
import zlib

from pathlib import Path
from uuid import uuid4


def ungzip_if_required(output):
    if isinstance(output, bytes) and output.startswith(b"\x1f\x8b"):
        return zlib.decompress(output, 16 + zlib.MAX_WBITS).decode('utf-8')

    return output


class ThreadedTCPServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
    pass


class FileHTTPServerRequestHandler(http.server.SimpleHTTPRequestHandler):
    def translate_path(self, path):
        """Translate a /-separated PATH to the local filename syntax.

        Components that mean special things to the local file system
        (e.g. drive or directory names) are ignored.  (XXX They should
        probably be diagnosed.)

        """
        # abandon query parameters
        path = path.split('?', 1)[0]
        path = path.split('#', 1)[0]
        path = posixpath.normpath(urllib.parse.unquote(path))
        words = path.split('/')
        words = [_f for _f in words if _f]
        path = self.rootPath
        for word in words:
            _, word = os.path.splitdrive(word)
            _, word = os.path.split(word)
            if word in (os.curdir, os.pardir):
                continue
            path = os.path.join(path, word)
        return path

    def log_message(self, format, *args):
        pass


class GPGFinder(object):
    """
    GnuPG binary discovery.
    """

    def __init__(self):
        self.gpg1 = self.find_gpg(["gpg1", "gpg"], "gpg (GnuPG) 1.")
        self.gpg2 = self.find_gpg(["gpg2", "gpg"], "gpg (GnuPG) 2.")

        self.gpg = self.gpg1
        if self.gpg is None:
            self.gpg = self.gpg2

        if self.gpg is None:
            raise Exception("GnuPG binary wasn't found")

    def find_gpg(self, executables, expected_version):
        for executable in executables:
            try:
                output = subprocess.check_output([executable, "--version"], text=True)
                if expected_version in output:
                    return executable
            except Exception:
                pass

        return None


class DotFinder(object):
    """
    dot binary discovery.
    """

    def __init__(self):
        self.dot = self.find_dot(["dot"])

    def find_dot(self, executables):
        for executable in executables:
            try:
                subprocess.check_output([executable, "-V"], text=True, stderr=subprocess.DEVNULL)
                return executable
            except Exception:
                pass

        return None


class BaseTest(object):
    """
    Base class for all tests.
    """

    skipTest = False
    longTest = False
    fixturePool = False
    fixturePoolCopy = False
    fixtureDB = False
    fixtureGpg = False
    fixtureWebServer = False
    requiresFTP = False
    requiresGPG1 = False
    requiresGPG2 = False
    requiresDot = False
    sortOutput = False
    debugOutput = False
    EtcdServer = None

    aptlyDir = ".aptly"
    aptlyConfigFile = ".aptly.conf"
    expectedCode = 0
    databaseType = ""
    databaseUrl = ""

    configOverride = {}
    environmentOverride = {}

    fixtureDBDir = os.path.join(os.environ["HOME"], "aptly-fixture-db")
    fixturePoolDir = os.path.join(os.environ["HOME"], "aptly-fixture-pool")
    fixtureGpgKeys = ["debian-archive-keyring.gpg",
                      "ubuntu-archive-keyring.gpg",
                      "launchpad.key",
                      "flat.key",
                      "pagerduty.key",
                      "nvidia.key",
                      "jenkins.key"]

    outputMatchPrepare = None

    captureResults = False

    gpgFinder = GPGFinder()
    dotFinder = DotFinder()

    def test(self):
        self.prepare()
        try:
            self.run()
            self.check()
        except Exception as exc:
            if self.debugOutput:
                print(f"API log:\n{self.debug_output()}")
            raise exc
        finally:
            self.teardown()

    def prepare_remove_all(self):
        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir)):
            shutil.rmtree(os.path.join(os.environ["HOME"], self.aptlyDir))
        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyConfigFile)):
            os.remove(os.path.join(os.environ["HOME"], self.aptlyConfigFile))
        if os.path.exists(os.path.join(os.environ["HOME"], ".gnupg", "aptlytest.gpg")):
            os.remove(os.path.join(
                os.environ["HOME"], ".gnupg", "aptlytest.gpg"))

    def prepare_default_config(self):
        databaseBackend = {
            "type": self.databaseType,
            "url": self.databaseUrl,
        }

        cfg = {
            "rootDir": f"{os.environ['HOME']}/{self.aptlyDir}",
            "downloadConcurrency": 4,
            "downloadSpeedLimit": 0,
            "downloadRetries": 5,
            "databaseOpenAttempts": 10,
            "architectures": [],
            "dependencyFollowSuggests": False,
            "dependencyFollowRecommends": False,
            "dependencyFollowAllVariants": False,
            "dependencyFollowSource": False,
            "gpgDisableVerify": False,
            "gpgDisableSign": False,
            "ppaDistributorID": "ubuntu",
            "ppaCodename": "",
            "enableMetricsEndpoint": True,
            "logLevel": "debug",
            "logFormat": "default",
            "serveInAPIMode": True,
            "databaseBackend": databaseBackend,
        }
        if self.requiresGPG1:
            cfg["gpgProvider"] = "gpg1"
        elif self.requiresGPG2:
            cfg["gpgProvider"] = "gpg2"
        cfg.update(**self.configOverride)
        f = open(os.path.join(os.environ["HOME"], self.aptlyConfigFile), "w")
        f.write(json.dumps(cfg))
        f.close()

    def fixture_available(self):
        if self.fixturePool and not os.path.exists(self.fixturePoolDir):
            return False
        if self.fixtureDB and not os.path.exists(self.fixtureDBDir):
            return False
        if self.requiresFTP and os.environ.get('NO_FTP_ACCESS', '') == 'yes':
            return False
        if self.requiresGPG1 and self.gpgFinder.gpg1 is None:
            return False
        if self.requiresGPG2 and self.gpgFinder.gpg2 is None:
            return False
        if self.requiresDot and self.dotFinder.dot is None:
            return False

        return True

    def prepare_fixture(self):
        if self.fixturePool:
            os.makedirs(os.path.join(os.environ["HOME"], self.aptlyDir), 0o755)
            os.symlink(self.fixturePoolDir, os.path.join(
                os.environ["HOME"], self.aptlyDir, "pool"))

        if self.fixturePoolCopy:
            os.makedirs(os.path.join(os.environ["HOME"], self.aptlyDir), 0o755)
            shutil.copytree(self.fixturePoolDir, os.path.join(
                os.environ["HOME"], self.aptlyDir, "pool"), ignore=shutil.ignore_patterns(".git"))

        if self.databaseType == "etcd":
            if not os.path.exists("/tmp/aptly-etcd"):
                self.run_cmd([os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "t13_etcd/install-etcd.sh")])

        if self.fixtureDB and self.databaseType != "etcd":
            shutil.copytree(self.fixtureDBDir, os.path.join(os.environ["HOME"], self.aptlyDir, "db"))

        if self.databaseType == "etcd":
            if self.EtcdServer:
                self.shutdown_etcd()

            # remove existing database
            if os.path.exists("/tmp/aptly-etcd-data"):
                shutil.rmtree("/tmp/aptly-etcd-data")

            if self.fixtureDB:
                print("import etcd")
                self.run_cmd(["/tmp/aptly-etcd/etcdctl", "--data-dir=/tmp/aptly-etcd-data", "snapshot", "restore", os.path.join(os.environ["HOME"], "etcd.db")])

            print("starting etcd")
            self.EtcdServer = self._start_process([os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "t13_etcd/start-etcd.sh")], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        if self.fixtureWebServer:
            self.webServerUrl = self.start_webserver(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)),
                                                                  self.fixtureWebServer))

        if self.requiresGPG2 or self.gpgFinder.gpg2:
            self.run_cmd([
                self.gpgFinder.gpg2, "--import",
                os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files") + "/aptly.sec"], expected_code=None)

        if self.fixtureGpg:
            self.run_cmd([self.gpgFinder.gpg, "--no-default-keyring", "--trust-model", "always", "--batch", "--keyring", "aptlytest.gpg", "--import"] +
                         [os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", key) for key in self.fixtureGpgKeys])
            self.run_cmd(["chmod", "400", os.path.join(os.environ["HOME"], ".gnupg/aptlytest.gpg")])

        if hasattr(self, "fixtureCmds"):
            for cmd in self.fixtureCmds:
                output = self.run_cmd(cmd)
                print("fixture Output:\n")
                for line in output.split("\n"):
                    print(f"    {line}")

    def sort_lines(self, output):
        return "\n".join(sorted(self.ensure_utf8(output).split("\n")))

    def run(self):
        output = self.run_cmd(self.runCmd, self.expectedCode)
        if self.sortOutput:
            output = self.sort_lines(output)
        self.output = self.output_processor(output)

    def _start_process(self, command, stderr=subprocess.STDOUT, stdout=None):
        if isinstance(command, str):
            params = {
                'files': os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files"),
                'changes': os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"),
                'udebs': os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "udebs"),
                'testfiles': os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__),
                'aptlyroot': os.path.join(os.environ["HOME"], self.aptlyDir),
            }
            if self.fixtureWebServer:
                params['url'] = self.webServerUrl

            command = string.Template(command).substitute(params)
            print(f"running command: {command}\n")
            command = shlex.split(command)

        if command[0] == "aptly":
            aptly_testing_bin = Path(__file__).parent / ".." / "aptly.test"
            command = [str(aptly_testing_bin), f"-test.coverprofile={Path(self.coverage_dir) / self.__class__.__name__}-{uuid4()}.out", *command[1:]]

        environ = os.environ.copy()
        environ["LC_ALL"] = "C"
        environ.update(self.environmentOverride)
        return subprocess.Popen(command, stderr=stderr, stdout=stdout, env=environ)

    def run_cmd(self, command, expected_code=0):
        proc = self._start_process(command, stdout=subprocess.PIPE)
        raw_output, _ = proc.communicate()

        raw_output = raw_output.decode("utf-8", errors='replace')

        returncodes = [proc.returncode]
        is_aptly_command = False
        if isinstance(command, str):
            is_aptly_command = command.startswith("aptly")

        if isinstance(command, list):
            is_aptly_command = command[0] == "aptly"

        if is_aptly_command:
            # remove the last two rows as go tests always print PASS/FAIL and coverage in those
            # two lines. This would otherwise fail the tests as they would not match gold
            matches = re.findall(r"((.|\n)*)EXIT: (\d)\n.*\ncoverage: .*", raw_output)
            if not matches:
                raise Exception("no matches found in command output '%s'" % raw_output)

            output, _, returncode = matches[0]
            returncodes.append(int(returncode))
        else:
            output = raw_output

        if expected_code is not None:
            if expected_code not in returncodes:
                raise Exception("command expected to return %d, but returned %d: \n%s" % (
                    expected_code, proc.returncode, raw_output))
        return output

    def gold_processor(self, gold):
        return gold

    def output_processor(self, output):
        return output

    def expand_environ(self, gold):
        return string.Template(gold).substitute(os.environ)

    def get_gold_filename(self, gold_name="gold"):
        return os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__ + "_" + gold_name)

    def get_gold(self, gold_name="gold"):
        return self.gold_processor(open(self.get_gold_filename(gold_name), "r").read())

    def strip_retry_lines(self, s):
        for prefix in (
                'Following redirect',
                'Error downloading',
                'Retrying',
        ):
            s = re.sub(r'{}.*\n'.format(prefix), '', s)
        return s

    def check_output(self):
        gold_file = self.get_gold_filename()
        print(f"Verifying gold file: {gold_file}")
        try:
            self.verify_match(self.get_gold(), self.output,
                              match_prepare=self.outputMatchPrepare)
        except Exception:  # noqa: E722
            if self.captureResults:
                if self.outputMatchPrepare is not None:
                    self.output = self.outputMatchPrepare(self.output)
                with open(self.get_gold_filename(), "w") as f:
                    f.write(self.output)
            else:
                raise

    def check_cmd_output(self, command, gold_name, match_prepare=None, expected_code=0):
        output = self.run_cmd(command, expected_code=expected_code)
        try:
            self.verify_match(self.get_gold(gold_name), output, match_prepare)
        except:  # noqa: E722
            if self.captureResults:
                if match_prepare is not None:
                    output = match_prepare(output)
                with open(self.get_gold_filename(gold_name), "w") as f:
                    f.write(output)
            else:
                raise

    def write_file(self, path, content):
        full_path = os.path.join(os.environ["HOME"], ".aptly", path)

        if not os.path.exists(os.path.dirname(full_path)):
            os.makedirs(os.path.dirname(full_path), 0o755)

        with open(full_path, "w") as f:
            f.write(content)

    def read_file(self, path, mode=''):
        with open(os.path.join(os.environ["HOME"], self.aptlyDir, path), "r" + mode) as f:
            return f.read()

    def delete_file(self, path):
        os.unlink(os.path.join(os.environ["HOME"], self.aptlyDir, path))

    def check_file_contents(self, path, gold_name, match_prepare=None, mode='', ensure_utf8=True):
        contents = self.read_file(path, mode=mode)
        try:

            self.verify_match(self.get_gold(gold_name),
                              contents, match_prepare=match_prepare,
                              ensure_utf8=ensure_utf8)
        except:  # noqa: E722
            if self.captureResults:
                if match_prepare is not None:
                    contents = match_prepare(contents)
                with open(self.get_gold_filename(gold_name), "w") as f:
                    f.write(contents)
            else:
                raise

    def check_file(self):
        contents = open(self.checkedFile, "r").read()
        try:
            self.verify_match(self.get_gold(), contents)
        except:  # noqa: E722
            if self.captureResults:
                with open(self.get_gold_filename(), "w") as f:
                    f.write(contents)
            else:
                raise

    def check_exists(self, path):
        if not os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir, path)):
            raise Exception("path %s doesn't exist" % (path, ))

    def check_not_exists(self, path):
        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir, path)):
            raise Exception("path %s exists" % (path, ))

    def check_file_not_empty(self, path):
        if os.stat(os.path.join(os.environ["HOME"], self.aptlyDir, path))[6] == 0:
            raise Exception("file %s is empty" % (path, ))

    def check_equal(self, a, b):
        if a != b:
            self.verify_match(a, b, match_prepare=pprint.pformat)

    def check_ge(self, a, b):
        if not a >= b:
            raise Exception("%s is not greater or equal to %s" % (a, b))

    def check_gt(self, a, b):
        if not a > b:
            raise Exception("%s is not greater to %s" % (a, b))

    def check_in(self, item, l):
        if item not in l:
            raise Exception("expected item: %r\nnot found in: %r" % (item, l))

    def check_not_in(self, item, l):
        if item in l:
            raise Exception("unexpected item: %r\n found in: %r" % (item, l))

    def check_subset(self, a, b):
        diff = ''
        for k, v in list(a.items()):
            if k not in b:
                diff += "unexpected key '%s'\n" % (k,)
            elif b[k] != v:
                diff += "wrong value '%s' for key '%s', expected '%s'\n" % (
                    v, k, b[k])
        if diff:
            raise Exception("content subset doesn't match:\n" + diff)

    def ensure_utf8(self, a):
        if isinstance(a, bytes):
            return a.decode('utf-8')
        return a

    def verify_match(self, a, b, match_prepare=None, ensure_utf8=True):
        if ensure_utf8:
            a = self.ensure_utf8(a)
            b = self.ensure_utf8(b)

        if match_prepare is not None:
            a = match_prepare(a)
            b = match_prepare(b)

        # strip trailing whitespace and newlines
        a = a.strip()
        b = b.strip()

        if a != b:
            diff = "".join(difflib.unified_diff(
                [l + "\n" for l in a.split("\n")], [l + "\n" for l in b.split("\n")]))

            raise Exception("content doesn't match:\n" + diff + "\n")

    check = check_output

    def prepare(self):
        self.prepare_remove_all()
        self.prepare_default_config()
        self.prepare_fixture()

    def teardown(self):
        if self.EtcdServer:
            self.shutdown_etcd()

    def start_webserver(self, directory):
        FileHTTPServerRequestHandler.rootPath = directory
        self.webserver = ThreadedTCPServer(
            ("localhost", 0), FileHTTPServerRequestHandler)

        server_thread = threading.Thread(target=self.webserver.serve_forever)
        server_thread.daemon = True
        server_thread.start()

        return "http://%s:%d/" % self.webserver.server_address

    def shutdown(self):
        if hasattr(self, 'webserver'):
            self.shutdown_webserver()
        if self.EtcdServer:
            self.shutdown_etcd()

    def shutdown_webserver(self):
        self.webserver.shutdown()

    def shutdown_etcd(self):
        print("stopping etcd")
        self.EtcdServer.terminate()
        self.EtcdServer.wait()
        self.EtcdServer = None

    @classmethod
    def shutdown_class(cls):
        pass



import tempfile
import shutil
import os
import inspect
from lib import BaseTest


class AddRepo1Test(BaseTest):
    """
    add package to local repo: .deb file
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo1 -distribution=squeeze repo1",
    ]
    runCmd = "aptly repo add repo1 ${files}/libboost-program-options-dev_1.49.0.1_i386.deb"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo1", "repo_show")

        # check pool
        self.check_exists('pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')


class AddRepo2Test(BaseTest):
    """
    add package to local repo: .dsc file
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo2 -distribution=squeeze repo2",
    ]
    runCmd = "aptly repo add repo2 ${files}/pyspi_0.6.1-1.3.dsc ${files}/pyspi-0.6.1-1.3.stripped.dsc"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo2", "repo_show")

        # check pool
        self.check_exists('pool/2e/77/0b28df948f3197ed0b679bdea99f_pyspi_0.6.1-1.3.diff.gz')
        self.check_exists('pool/d4/94/aaf526f1ec6b02f14c2f81e060a5_pyspi_0.6.1-1.3.dsc')
        self.check_exists('pool/64/06/9ee828c50b1c597d10a3fefbba27_pyspi_0.6.1.orig.tar.gz')
        self.check_exists('pool/28/9d/3aefa970876e9c43686ce2b02f47_pyspi-0.6.1-1.3.stripped.dsc')


class AddRepo3Test(BaseTest):
    """
    add package to local repo: directory
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo3 -distribution=squeeze repo3",
    ]
    runCmd = "aptly repo add repo3 ${files}"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo3", "repo_show")

        # check pool
        self.check_exists('pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')
        self.check_exists('pool/2e/77/0b28df948f3197ed0b679bdea99f_pyspi_0.6.1-1.3.diff.gz')
        self.check_exists('pool/d4/94/aaf526f1ec6b02f14c2f81e060a5_pyspi_0.6.1-1.3.dsc')
        self.check_exists('pool/64/06/9ee828c50b1c597d10a3fefbba27_pyspi_0.6.1.orig.tar.gz')
        self.check_exists('pool/28/9d/3aefa970876e9c43686ce2b02f47_pyspi-0.6.1-1.3.stripped.dsc')


class AddRepo4Test(BaseTest):
    """
    add package to local repo: complex directory + remove
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo4 -distribution=squeeze repo4",
    ]
    runCmd = "aptly repo add -remove-files repo4 "

    def prepare(self):
        super(AddRepo4Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()
        os.makedirs(os.path.join(self.tempSrcDir, "01"), 0o755)
        os.makedirs(os.path.join(self.tempSrcDir, "02", "03"), 0o755)

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "libboost-program-options-dev_1.49.0.1_i386.deb"),
                    os.path.join(self.tempSrcDir, "01"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.dsc"),
                    os.path.join(self.tempSrcDir, "02", "03"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1.orig.tar.gz"),
                    os.path.join(self.tempSrcDir, "02", "03"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.diff.gz"),
                    os.path.join(self.tempSrcDir, "02", "03"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.diff.gz"),
                    os.path.join(self.tempSrcDir, "02", "03", "other.file"))

        self.runCmd += self.tempSrcDir

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo4", "repo_show")

        # check pool
        self.check_exists('pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')
        self.check_exists('pool/2e/77/0b28df948f3197ed0b679bdea99f_pyspi_0.6.1-1.3.diff.gz')
        self.check_exists('pool/d4/94/aaf526f1ec6b02f14c2f81e060a5_pyspi_0.6.1-1.3.dsc')
        self.check_exists('pool/64/06/9ee828c50b1c597d10a3fefbba27_pyspi_0.6.1.orig.tar.gz')

        path = os.path.join(self.tempSrcDir, "01", "libboost-program-options-dev_1.49.0.1_i386.deb")
        if os.path.exists(path):
            raise Exception("path %s shouldn't exist" % (path, ))
        path = os.path.join(self.tempSrcDir, "02", "03", "pyspi_0.6.1.orig.tar.gz")
        if os.path.exists(path):
            raise Exception("path %s shouldn't exist" % (path, ))

        path = os.path.join(self.tempSrcDir, "02", "03", "other.file")
        if not os.path.exists(path):
            raise Exception("path %s doesn't exist" % (path, ))

        shutil.rmtree(self.tempSrcDir)


class AddRepo5Test(BaseTest):
    """
    add package to local repo: some source files missing
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo5 -distribution=squeeze repo5",
    ]
    runCmd = "aptly repo add repo5 "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return s.replace(self.tempSrcDir, "")

    def prepare(self):
        super(AddRepo5Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()
        os.makedirs(os.path.join(self.tempSrcDir, "02", "03"), 0o755)

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.dsc"),
                    os.path.join(self.tempSrcDir, "02", "03"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1.orig.tar.gz"),
                    os.path.join(self.tempSrcDir, "02", "03"))

        self.runCmd += self.tempSrcDir

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo5", "repo_show")

        shutil.rmtree(self.tempSrcDir)


class AddRepo6Test(BaseTest):
    """
    add package to local repo: missing file
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo6 -distribution=squeeze repo6",
    ]
    runCmd = "aptly repo add repo6 no-such-file"
    expectedCode = 1


class AddRepo7Test(BaseTest):
    """
    add package to local repo: missing repo
    """
    runCmd = "aptly repo add repo7 ${files}"
    expectedCode = 1


class AddRepo8Test(BaseTest):
    """
    add package to local repo: conflict in packages
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo8 -distribution=squeeze repo8",
        "aptly repo add repo8 ${files}/pyspi_0.6.1-1.3.dsc",
    ]
    runCmd = "aptly repo add repo8 ${testfiles}/pyspi_0.6.1-1.3.conflict.dsc"
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__), ""). \
                         replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files"), "")

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo8", "repo_show")


class AddRepo9Test(BaseTest):
    """
    add package to local repo: conflict in files
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo9 -distribution=squeeze repo9",
    ]
    runCmd = "aptly repo add repo9 ${files}/pyspi_0.6.1-1.3.dsc"
    gold_processor = BaseTest.expand_environ
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__), ""). \
                         replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files"), "")

    def prepare(self):
        super(AddRepo9Test, self).prepare()

        os.makedirs(os.path.join(os.environ["HOME"], ".aptly", "pool/64/06/"))
        with open(os.path.join(os.environ["HOME"], ".aptly", "pool/64/06/9ee828c50b1c597d10a3fefbba27_pyspi_0.6.1.orig.tar.gz"), "w") as f:
            f.write("abcd")


class AddRepo10Test(BaseTest):
    """
    add package to local repo: double import
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo10 -distribution=squeeze repo10",
        "aptly repo add repo10 ${files}",
    ]
    runCmd = "aptly repo add repo10 ${files}/pyspi_0.6.1-1.3.dsc"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo10", "repo_show")


class AddRepo11Test(BaseTest):
    """
    add package to local repo: conflict in packages + -force-replace
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo11 -distribution=squeeze repo11",
        "aptly repo add repo11 ${files}/pyspi_0.6.1-1.3.dsc",
    ]
    runCmd = "aptly repo add -force-replace repo11 ${testfiles}/pyspi_0.6.1-1.3.conflict.dsc"

    def outputMatchPrepare(self, s):
        return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__), ""). \
                         replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files"), "")

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo11", "repo_show")


class AddRepo12Test(BaseTest):
    """
    add package to local repo: .udeb file
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo12 -distribution=squeeze repo12",
    ]
    runCmd = "aptly repo add repo12 ${udebs}/dmraid-udeb_1.0.0.rc16-4.1_amd64.udeb"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo12", "repo_show")

        # check pool
        self.check_exists('pool/ef/ae/69921b97494e40437712053b60a5_dmraid-udeb_1.0.0.rc16-4.1_amd64.udeb')


class AddRepo13Test(BaseTest):
    """
    add package to local repo: .udeb and .deb files
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo13 -distribution=squeeze repo13",
    ]
    runCmd = "aptly repo add repo13 ${udebs} ${files}"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo13", "repo_show")

        # check pool
        self.check_exists('pool/ef/ae/69921b97494e40437712053b60a5_dmraid-udeb_1.0.0.rc16-4.1_amd64.udeb')
        self.check_exists('pool/d4/94/aaf526f1ec6b02f14c2f81e060a5_pyspi_0.6.1-1.3.dsc')


class AddRepo14Test(BaseTest):
    """
    add same package to local repo twice and make sure the file doesn't get truncated.
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo14 -distribution=squeeze repo14",
        "aptly repo add repo14 ${files}/libboost-program-options-dev_1.49.0.1_i386.deb",
        "aptly publish repo -distribution=test1 -skip-signing repo14"
    ]
    runCmd = "aptly repo add repo14 $aptlyroot/public/pool/"

    def check(self):
        super(AddRepo14Test, self).check()
        # check pool
        self.check_exists('pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')


class AddRepo15Test(BaseTest):
    """
    add package with wrong case in stanza and missing fields
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo15 -distribution=squeeze repo15",
    ]
    runCmd = "aptly repo add repo15 ${testfiles}"
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__), ""). \
                         replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files"), "")


class AddRepo16Test(BaseTest):
    """
    add package to local repo: some source files missing, but already in the pool
    """
    fixtureCmds = [
        "aptly repo create repo1",
        "aptly repo create repo2",
        "aptly repo add repo1 ${files}"
    ]
    runCmd = "aptly repo add repo2 "

    def outputMatchPrepare(self, s):
        return s.replace(self.tempSrcDir, "")

    def prepare(self):
        super(AddRepo16Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()
        os.makedirs(os.path.join(self.tempSrcDir, "02", "03"), 0o755)

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.dsc"),
                    os.path.join(self.tempSrcDir, "02", "03"))
        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1.orig.tar.gz"),
                    os.path.join(self.tempSrcDir, "02", "03"))

        self.runCmd += self.tempSrcDir

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo2", "repo_show")

        shutil.rmtree(self.tempSrcDir)


class AddRepo17Test(BaseTest):
    """
    add packages to local repo: .deb file with provides
    """
    fixtureCmds = [
        "aptly repo create -comment=Repo17 -distribution=squeeze repo17",
    ]
    runCmd = "aptly repo add repo17 ${testfiles}/mesa-stable_24.2.6-101pika1_amd64.deb ${testfiles}/mesa-stable-no-march_24.2.6-101pika1_amd64.deb"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo17", "repo_show")

        # check pool
        self.check_exists('pool/e3/f6/51297bd4bd0ef999296ef0a28299_mesa-stable-no-march_24.2.6-101pika1_amd64.deb')
        self.check_exists('pool/01/6b/3d864229761eff49a8680c9987ab_mesa-stable_24.2.6-101pika1_amd64.deb')



import inspect
import os
import subprocess
import tempfile
import json

from api_lib import APITest


def check_gpgkey_exists(gpg_key, keyring):
    p = subprocess.Popen([
        "gpg", "--no-default-keyring",
        "--keyring", keyring,
        "--fingerprint", gpg_key],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    p.communicate()
    if p.returncode != 0:
        raise Exception("gpg key does not exists")


class GPGAPITestAddKey(APITest):
    """
    POST /gpg/key
    """
    requiresGPG2 = True

    fixtureCmds = [
        "gpgconf --kill dirmngr",
        "gpgconf --launch dirmngr",
        "sleep 2"
    ]

    def check(self):
        with tempfile.NamedTemporaryFile(suffix=".pub") as keyring:
            gpgkeyid = "9E3E53F19C7DE460"
            resp = self.post("/api/gpg/key", json={
                "Keyserver": "hkp://keyserver.ubuntu.com:80",
                "Keyring": keyring.name,
                "GpgKeyID": gpgkeyid
            })
            if resp.status_code != 200:
                output = json.loads(resp.text)
                print(f"{output}\n")
            self.check_equal(resp.status_code, 200)
            check_gpgkey_exists(gpgkeyid, keyring.name)


class GPGAPITestAddKeyArmor(APITest):
    """
    POST /gpg/key
    """
    def check(self):
        keyfile = os.path.join(os.path.dirname(inspect.getsourcefile(APITest)),
                               "files") + "/launchpad.key"
        gpgkeyid = "3B1F56C0"

        with open(keyfile, 'r') as keyf:
            gpgkeyarmor = keyf.read()

        with tempfile.NamedTemporaryFile(suffix=".pub") as keyring:
            resp = self.post("/api/gpg/key", json={
                "Keyring": keyring.name,
                "GpgKeyArmor": gpgkeyarmor
            })

            self.check_equal(resp.status_code, 200)
            check_gpgkey_exists(gpgkeyid, keyring.name)



from api_lib import APITest

from .publish import DefaultSigningOptions


class SnapshotsAPITestCreateShowEmpty(APITest):
    """
    GET /api/snapshots/:name, POST /api/snapshots, GET /api/snapshots/:name/packages
    """

    def check(self):
        snapshot_name = self.random_name()
        snapshot_desc = {'Description': 'fun snapshot',
                         'Name': snapshot_name}

        # create empty snapshot
        task = self.post_task("/api/snapshots", json=snapshot_desc)
        self.check_task(task)

        self.check_subset(snapshot_desc, self.get("/api/snapshots/" + snapshot_name).json())
        self.check_equal(self.get("/api/snapshots/" + snapshot_name).status_code, 200)

        resp = self.get("/api/snapshots/" + snapshot_name + "/packages")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [])

        self.check_equal(self.get("/api/snapshots/" + self.random_name()).status_code, 404)

        # create snapshot with duplicate name
        task = self.post_task("/api/snapshots", json=snapshot_desc)
        self.check_task_fail(task)


class SnapshotsAPITestCreateFromRefs(APITest):
    """
    GET /api/snapshots/:name, POST /api/snapshots, GET /api/snapshots/:name/packages,
    GET /api/snapshots
    """

    def check(self):
        snapshot_name = self.random_name()
        snapshot_desc = {'Description': 'fun snapshot',
                         'Name': snapshot_name,
                         'SourceSnapshots': [self.random_name()]}

        # creating snapshot from missing source snapshot
        resp = self.post("/api/snapshots", json=snapshot_desc)
        self.check_equal(resp.status_code, 404)

        # create empty snapshot
        empty_snapshot_name = self.random_name()
        task = self.post_task("/api/snapshots", json={"Name": empty_snapshot_name})
        self.check_task(task)
        self.check_equal(
            self.get("/api/snapshots/" + empty_snapshot_name).json()['Description'], "Created as empty"
        )

        # create and upload package to repo to register package in DB
        repo_name = self.random_name()
        self.check_equal(self.post("/api/repos", json={"Name": repo_name}).status_code, 201)
        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        # create snapshot with empty snapshot as source and package
        snapshot = snapshot_desc.copy()
        snapshot['PackageRefs'] = ["Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378"]
        snapshot['SourceSnapshots'] = [empty_snapshot_name]
        task = self.post_task("/api/snapshots", json=snapshot)
        self.check_task(task)
        snapshot.pop('SourceSnapshots')
        snapshot.pop('PackageRefs')
        resp = self.get("/api/snapshots/" + snapshot_name)
        self.check_equal(resp.status_code, 200)
        self.check_subset(snapshot, resp.json())

        self.check_subset(snapshot, self.get("/api/snapshots/" + snapshot_name).json())
        resp = self.get("/api/snapshots/" + snapshot_name + "/packages")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), ["Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378"])

        # create snapshot with unreferenced package
        task = self.post_task("/api/snapshots", json={
            "Name": self.random_name(),
            "PackageRefs": ["Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378", "Pamd64 no-such-package 1.2 91"]})
        self.check_task_fail(task)

        # list snapshots
        resp = self.get("/api/snapshots", params={"sort": "time"})
        self.check_equal(resp.status_code, 200)
        self.check_equal([s["Name"] for s in resp.json() if s["Name"] in [empty_snapshot_name, snapshot_name]],
                         [empty_snapshot_name, snapshot_name])


class SnapshotsAPITestCreateFromRepo(APITest):
    """
    POST /api/repos, POST /api/repos/:name/snapshots, GET /api/snapshots/:name
    """

    def check(self):
        repo_name = self.random_name()
        snapshot_name = self.random_name()
        self.check_equal(self.post("/api/repos", json={"Name": repo_name}).status_code, 201)

        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot_name})
        self.check_task(task)
        self.check_equal([],
                         self.get("/api/snapshots/" + snapshot_name + "/packages", params={"format": "details"}).json())

        snapshot_name = self.random_name()
        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot_name})
        self.check_task(task)
        self.check_equal(self.get("/api/snapshots/" + snapshot_name).status_code, 200)

        self.check_subset({'Architecture': 'i386',
                           'Package': 'libboost-program-options-dev',
                           'Version': '1.49.0.1',
                           'FilesHash': '918d2f433384e378'},
                          self.get("/api/snapshots/" + snapshot_name + "/packages", params={"format": "details"}).json()[0])

        self.check_subset({'Architecture': 'i386',
                           'Package': 'libboost-program-options-dev',
                           'Version': '1.49.0.1',
                           'FilesHash': '918d2f433384e378'},
                          self.get("/api/snapshots/" + snapshot_name + "/packages",
                                   params={"format": "details", "q": "Version (> 0.6.1-1.4)"}).json()[0])

        # duplicate snapshot name
        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot_name})
        self.check_task_fail(task)


class SnapshotsAPITestCreateUpdate(APITest):
    """
    POST /api/snapshots, PUT /api/snapshots/:name, GET /api/snapshots/:name
    """

    def check(self):
        snapshot_name = self.random_name()
        snapshot_desc = {'Description': 'fun snapshot',
                         'Name': snapshot_name}

        task = self.post_task("/api/snapshots", json=snapshot_desc)
        self.check_task(task)

        new_snapshot_name = self.random_name()
        task = self.put_task("/api/snapshots/" + snapshot_name, json={'Name': new_snapshot_name,
                                                                      'Description': 'New description'})
        self.check_task(task)

        resp = self.get("/api/snapshots/" + new_snapshot_name)
        self.check_equal(resp.status_code, 200)
        self.check_subset({"Name": new_snapshot_name,
                           "Description": "New description"}, resp.json())

        # duplicate name
        task = self.put_task("/api/snapshots/" + new_snapshot_name, json={'Name': new_snapshot_name,
                                                                          'Description': 'New description'})
        self.check_task_fail(task)

        # missing snapshot
        resp = self.put("/api/snapshots/" + snapshot_name, json={})
        self.check_equal(resp.status_code, 404)


class SnapshotsAPITestCreateDelete(APITest):
    """
    POST /api/snapshots, DELETE /api/snapshots/:name, GET /api/snapshots/:name
    """

    def check(self):
        snapshot_name = self.random_name()
        snapshot_desc = {'Description': 'fun snapshot',
                         'Name': snapshot_name}

        # deleting unreferenced snapshot
        task = self.post_task("/api/snapshots", json=snapshot_desc)
        self.check_task(task)

        task = self.delete_task("/api/snapshots/" + snapshot_name)
        self.check_task(task)

        self.check_equal(self.get("/api/snapshots/" + snapshot_name).status_code, 404)

        # deleting referenced snapshot
        snap1, snap2 = self.random_name(), self.random_name()
        task = self.post_task("/api/snapshots", json={"Name": snap1})
        self.check_task(task)
        self.check_equal(
            self.post_task(
                "/api/snapshots", json={"Name": snap2, "SourceSnapshots": [snap1]}
            ).json()['State'], 2
        )

        task = self.delete_task("/api/snapshots/" + snap1)
        self.check_task_fail(task)
        self.check_equal(self.get("/api/snapshots/" + snap1).status_code, 200)
        task = self.delete_task("/api/snapshots/" + snap1, params={"force": "1"})
        self.check_task(task)
        self.check_equal(self.get("/api/snapshots/" + snap1).status_code, 404)

        # deleting published snapshot
        task = self.post_task(
            "/api/publish",
            json={
                 "SourceKind": "snapshot",
                 "Distribution": "trusty",
                 "Architectures": ["i386"],
                 "Sources": [{"Name": snap2}],
                 "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)

        task = self.delete_task("/api/snapshots/" + snap2)
        self.check_task_fail(task)
        task = self.delete_task("/api/snapshots/" + snap2, params={"force": "1"})
        self.check_task_fail(task)


class SnapshotsAPITestSearch(APITest):
    """
    POST /api/snapshots, GET /api/snapshots?sort=name, GET /api/snapshots/:name
    """

    def check(self):

        repo_name = self.random_name()
        self.check_equal(self.post("/api/repos", json={"Name": repo_name}).status_code, 201)

        d = self.random_name()
        snapshot_name = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot_name})
        self.check_task(task)

        resp = self.get("/api/snapshots/" + snapshot_name + "/packages",
                        params={"q": "libboost-program-options-dev", "format": "details"})
        self.check_equal(resp.status_code, 200)

        self.check_equal(len(resp.json()), 1)
        self.check_equal(resp.json()[0]["Package"], "libboost-program-options-dev")

        resp = self.get("/api/snapshots/" + snapshot_name + "/packages")
        self.check_equal(resp.status_code, 200)

        self.check_equal(len(resp.json()), 1)
        self.check_equal(resp.json(), ["Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378"])


class SnapshotsAPITestDiff(APITest):
    """
    GET /api/snapshot/:name/diff/:name2
    """

    def check(self):
        repos = [self.random_name() for x in range(2)]
        snapshots = [self.random_name() for x in range(2)]

        for repo_name in repos:
            self.check_equal(self.post("/api/repos", json={"Name": repo_name}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        task = self.post_task("/api/repos/" + repos[-1] + "/file/" + d)
        self.check_task(task)

        task = self.post_task("/api/repos/" + repos[-1] + '/snapshots', json={'Name': snapshots[0]})
        self.check_task(task)

        task = self.post_task("/api/snapshots", json={'Name': snapshots[1]})
        self.check_task(task)

        resp = self.get("/api/snapshots/" + snapshots[0] + "/diff/" + snapshots[1])
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [{'Left': 'Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378',
                                        'Right': None}])

        resp = self.get("/api/snapshots/" + snapshots[1] + "/diff/" + snapshots[0])
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [{'Right': 'Pi386 libboost-program-options-dev 1.49.0.1 918d2f433384e378',
                                        'Left': None}])

        resp = self.get("/api/snapshots/" + snapshots[0] + "/diff/" + snapshots[0])
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [])

        resp = self.get("/api/snapshots/" + snapshots[1] + "/diff/" + snapshots[1])
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [])


class SnapshotsAPITestMerge(APITest):
    """
    POST /api/snapshots, POST /api/snapshots/:name/merge, GET /api/snapshots/:name, DELETE /api/snapshots/:name
    """

    def check(self):
        sources = [
            {"Description": "fun snapshot", "Name": self.random_name()}
            for _ in range(2)
        ]

        # create source snapshots
        for source in sources:
            task = self.post_task("/api/snapshots", json=source)
            self.check_task(task)

        # create merge snapshot
        merged_name = self.random_name()
        task = self.post_task(
            f"/api/snapshots/{merged_name}/merge",
            json={
                "Sources": [source["Name"] for source in sources],
            },
        )
        self.check_task(task)

        # check merge snapshot
        resp = self.get(f"/api/snapshots/{merged_name}")
        self.check_equal(resp.status_code, 200)
        source_list = ", ".join(f"'{source['Name']}'" for source in sources)
        self.check_subset(
            {
                "Name": merged_name,
                "Description": f"Merged from sources: {source_list}",
            },
            resp.json(),
        )

        # remove merge snapshot
        task = self.delete_task(f"/api/snapshots/{merged_name}")
        self.check_task(task)

        # create merge snapshot without sources
        merged_name = self.random_name()
        resp = self.post(
            f"/api/snapshots/{merged_name}/merge", json={"Sources": []}
        )
        self.check_equal(resp.status_code, 400)
        self.check_equal(
            resp.json()["error"], "At least one source snapshot is required"
        )
        self.check_equal(self.get(f"/api/snapshots/{merged_name}").status_code, 404)

        # create merge snapshot with non-existing source
        merged_name = self.random_name()
        non_existing_source = self.random_name()
        resp = self.post(
            f"/api/snapshots/{merged_name}/merge",
            json={"Sources": [non_existing_source]},
        )
        self.check_equal(
            resp.json()["error"], f"snapshot with name {non_existing_source} not found"
        )
        self.check_equal(resp.status_code, 404)

        self.check_equal(self.get(f"/api/snapshots/{merged_name}").status_code, 404)

        # create merge snapshot with used name
        merged_name = sources[0]["Name"]
        resp = self.post(
            f"/api/snapshots/{merged_name}/merge",
            json={"Sources": [source["Name"] for source in sources]},
        )
        self.check_equal(
            resp.json()["error"],
            f"unable to create snapshot: snapshot with name {sources[0]['Name']} already exists",
        )
        self.check_equal(resp.status_code, 500)

        # create merge snapshot with "latest" and "no-remove" flags (should fail)
        merged_name = self.random_name()
        resp = self.post(
            f"/api/snapshots/{merged_name}/merge",
            json={
                "Sources": [source["Name"] for source in sources],
            },
            params={"latest": "1", "no-remove": "1"},
        )
        self.check_equal(
            resp.json()["error"], "no-remove and latest are mutually exclusive"
        )
        self.check_equal(resp.status_code, 400)


class SnapshotsAPITestPull(APITest):
    """
    POST /api/snapshots/:name/pull, POST /api/snapshots, GET /api/snapshots/:name/packages?name=:package_name
    """

    def check(self):
        repo_with_libboost = self.random_name()
        empty_repo = self.random_name()
        snapshot_repo_with_libboost = self.random_name()
        snapshot_empty_repo = self.random_name()

        # create repo with file in it and snapshot of it
        self.check_equal(self.post("/api/repos", json={"Name": repo_with_libboost}).status_code, 201)

        dir_name = self.random_name()
        self.check_equal(self.upload(f"/api/files/{dir_name}",
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        self.check_equal(self.post(f"/api/repos/{repo_with_libboost}/file/{dir_name}").status_code, 200)

        resp = self.post(f"/api/repos/{repo_with_libboost}/snapshots", json={'Name': snapshot_repo_with_libboost})
        self.check_equal(resp.status_code, 201)

        # create empty repo and snapshot of it
        self.check_equal(self.post("/api/repos", json={"Name": empty_repo}).status_code, 201)

        resp = self.post(f"/api/repos/{empty_repo}/snapshots", json={'Name': snapshot_empty_repo})
        self.check_equal(resp.status_code, 201)

        # pull libboost from repo_with_libboost to empty_repo, save into snapshot_pull_libboost
        snapshot_pull_libboost = self.random_name()

        # dry run first
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull?dry-run=1", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ],
            'Architectures': [
                'amd64'
                'i386'
            ]
        })
        self.check_equal(resp.status_code, 200)

        # dry run, all-matches
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull?dry-run=1&all-matches=1", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ],
            'Architectures': [
                'amd64'
                'i386'
            ]
        })
        self.check_equal(resp.status_code, 200)

        # missing argument
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
        })
        self.check_equal(resp.status_code, 400)

        # dry run, emtpy architectures
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull?dry-run=1", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ],
            'Architectures': []
        })
        self.check_equal(resp.status_code, 500)

        # dry run, non-existing To
        resp = self.post("/api/snapshots/asd123/pull", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ]
        })
        self.check_equal(resp.status_code, 404)

        # dry run, non-existing source
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull?dry-run=1", json={
            'Source': "asd123",
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ]
        })
        self.check_equal(resp.status_code, 404)

        # snapshot pull
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull", json={
            'Source': snapshot_repo_with_libboost,
            'Destination': snapshot_pull_libboost,
            'Queries': [
                'libboost-program-options-dev'
            ],
            'Architectures': [
                'amd64'
                'i386'
            ]
        })
        self.check_equal(resp.status_code, 201)
        self.check_subset({
            'Name': snapshot_pull_libboost,
            'SourceKind': 'snapshot',
            'Description': f"Pulled into '{snapshot_empty_repo}' with '{snapshot_repo_with_libboost}' as source, pull request was: 'libboost-program-options-dev'",
        }, resp.json())

        # check that snapshot_pull_libboost contains libboost
        resp = self.get(f"/api/snapshots/{snapshot_pull_libboost}/packages?name=libboost-program-options-dev")
        self.check_equal(resp.status_code, 200)

        # pull from non-existing source
        non_existing_source = self.random_name()
        destination = self.random_name()
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull", json={
            'Source': non_existing_source,
            'Destination': destination,
            'Queries': [
                'Name (~ *)'
            ],
            'Architectures': [
                'all',
            ]
        })
        self.check_equal(resp.status_code, 404)
        self.check_equal(resp.json()['error'], f"snapshot with name {non_existing_source} not found")

        # pull to non-existing snapshot
        non_existing_snapshot = self.random_name()
        destination = self.random_name()
        resp = self.post(f"/api/snapshots/{snapshot_empty_repo}/pull", json={
            'Source': non_existing_snapshot,
            'Destination': destination,
            'Queries': [
                'Name (~ *)'
            ],
            'Architectures': [
                'all',
            ]
        })
        self.check_equal(resp.status_code, 404)
        self.check_equal(resp.json()['error'], f"snapshot with name {non_existing_snapshot} not found")



"""
Testing serving public repo
"""

import http.client
import os
import signal
import subprocess
import shlex
import time

from lib import BaseTest


class RootDirInaccessible(BaseTest):
    """
    serve command aborts if rootDir is inaccessible
    """
    skipTest = 'User is root' if os.environ['USER'] == 'root' else False
    fixtureDB = False
    fixturePool = False

    configOverride = {
        "rootDir": "/root"  # any directory that exists but is not writable
    }

    runCmd = "aptly serve -listen=127.0.0.1:8765"
    expectedCode = 1


class Serve1Test(BaseTest):
    """
    serve public: two publishes, verify HTTP
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 from mirror gnuplot-maverick-src",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap2 debian",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -component=main,contrib snap1 snap2 multi",
    ]
    runCmd = "../aptly.test serve -listen=127.0.0.1:8765"

    def run(self):
        try:
            proc = subprocess.Popen(shlex.split(self.runCmd), stderr=subprocess.STDOUT, stdout=subprocess.PIPE, bufsize=0)

            try:
                time.sleep(1)

                conn = http.client.HTTPConnection("127.0.0.1", 8765)
                conn.request("GET", "/")
                r = conn.getresponse()
                if r.status != 200:
                    raise Exception("Expected status 200 != %d" % r.status)
                self.http_response = r.read()

                output = os.read(proc.stdout.fileno(), 8192)

            finally:
                proc.send_signal(signal.SIGINT)
                proc.wait()

            if proc.returncode != -2 and proc.returncode != 2:
                raise Exception("exit code %d != %d (output: %s)" % (proc.returncode, -2, output))
            self.output = output
        except Exception as e:
            raise Exception("Running command %s failed: %s" % (self.runCmd, str(e)))

    def check(self):
        self.check_output()
        self.verify_match(self.get_gold('http'), self.http_response, match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))


class Serve2Test(BaseTest):
    """
    serve public: no publishes
    """
    runCmd = "aptly serve -listen=127.0.0.1:8765"



import urllib.error
import urllib.parse
import urllib.request

from api_lib import APITest


class PackagesAPITestShow(APITest):
    """
    GET /api/packages/:key
    """
    def check(self):
        # upload package to repo to create it
        repo_name = self.random_name()
        self.check_equal(self.post("/api/repos", json={"Name": repo_name, "Comment": "fun repo"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "pyspi_0.6.1-1.3.dsc", "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz").status_code, 200)

        resp = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(resp)

        # get information about package
        pyspi_json = {
            'Architecture': 'any',
            'Binary': 'python-at-spi',
            'Build-Depends': 'debhelper (>= 5), cdbs, libatspi-dev, python-pyrex, python-support (>= 0.4), python-all-dev, libx11-dev',  # noqa
            'Checksums-Sha1': ' 95a2468e4bbce730ba286f2211fa41861b9f1d90 3456 pyspi_0.6.1-1.3.diff.gz\n 56c8a9b1f4ab636052be8966690998cbe865cd6c 1782 pyspi_0.6.1-1.3.dsc\n 9694b80acc171c0a5bc99f707933864edfce555e 29063 pyspi_0.6.1.orig.tar.gz\n',  # noqa
            'Checksums-Sha256': ' 2e770b28df948f3197ed0b679bdea99f3f2bf745e9ddb440c677df9c3aeaee3c 3456 pyspi_0.6.1-1.3.diff.gz\n d494aaf526f1ec6b02f14c2f81e060a5722d6532ddc760ec16972e45c2625989 1782 pyspi_0.6.1-1.3.dsc\n 64069ee828c50b1c597d10a3fefbba279f093a4723965388cdd0ac02f029bfb9 29063 pyspi_0.6.1.orig.tar.gz\n',  # noqa
            'Checksums-Sha512': ' 384b5e94b4113262e41bda1a2563f4f439cb8c97f43e2caefe16d7626718c21b36d3145b915eed24053eaa7fe3b6186494a87a3fcf9627f6e653b54bb3caa897 3456 pyspi_0.6.1-1.3.diff.gz\n fde06b7dc5762a04986d0669420822f6a1e82b195322ae9cbd2dae40bda557c57ad77fe3546007ea645f801c4cd30ef4eb0e96efb2dee6b71c4c9a187d643683 1782 pyspi_0.6.1-1.3.dsc\n c278f52953203292bcc828bcf05aee456b160f91716f51ec1a1dbbcdb8b08fc29183d0a1135629fc0ebe86a3e84cedc685c3aa1714b70cc5db8877d40e754d7f 29063 pyspi_0.6.1.orig.tar.gz\n',  # noqa
            'Files': ' 22ff26db69b73d3438fdde21ab5ba2f1 3456 pyspi_0.6.1-1.3.diff.gz\n b72cb94699298a117b7c82641c68b6fd 1782 pyspi_0.6.1-1.3.dsc\n def336bd566ea688a06ec03db7ccf1f4 29063 pyspi_0.6.1.orig.tar.gz\n',  # noqa
            'FilesHash': '3a8b37cbd9a3559e',
            'Format': '1.0',
            'Homepage': 'http://people.redhat.com/zcerza/dogtail',
            'Key': 'Psource pyspi 0.6.1-1.3 3a8b37cbd9a3559e',
            'Maintainer': 'Jose Carlos Garcia Sogo <jsogo@debian.org>',
            'Package': 'pyspi',
            'ShortKey': 'Psource pyspi 0.6.1-1.3',
            'Standards-Version': '3.7.3',
            'Vcs-Svn': 'svn://svn.tribulaciones.org/srv/svn/pyspi/trunk',
            'Version': '0.6.1-1.3'
        }

        resp = self.get("/api/packages/" + urllib.parse.quote('Psource pyspi 0.6.1-1.3 3a8b37cbd9a3559e'))
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), pyspi_json)

        resp = self.get("/api/packages?q=pyspi")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [pyspi_json["Key"]])

        resp = self.get("/api/packages?q=pyspi&format=details")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [pyspi_json])

        resp = self.get("/api/packages/" + urllib.parse.quote('Pamd64 no-such-package 1.0 3a8b37cbd9a3559e'))
        self.check_equal(resp.status_code, 404)

        resp = self.get("/api/packages?q=no-such-package")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [])



import inspect
import json
import os
import random
import shutil
import string
import time

from lib import BaseTest
from testout import TestOut

try:
    import requests
except ImportError:
    requests = None

# States for returned tasks from the API
TASK_SUCCEEDED = 2
TASK_FAILED = 3


class APITest(BaseTest):
    """
    BaseTest + testing aptly API
    """
    aptly_server = None
    aptly_out = None
    debugOutput = True
    base_url = "127.0.0.1:8765"
    configOverride = {
        "FileSystemPublishEndpoints": {
            "apiandserve": {
                "rootDir": f"{os.environ['HOME']}/{BaseTest.aptlyDir}/apiandserve",
                "linkMethod": "symlink"
            }
        },
        "enableSwaggerEndpoint": True
    }

    def fixture_available(self):
        return super(APITest, self).fixture_available() and requests is not None

    def prepare(self):
        if APITest.aptly_server is None:
            super(APITest, self).prepare()

            APITest.aptly_out = TestOut()

            configPath = os.path.join(os.environ["HOME"], self.aptlyConfigFile)
            APITest.aptly_server = self._start_process(f"aptly api serve -no-lock -config={configPath} -listen={self.base_url}", stdout=APITest.aptly_out, stderr=APITest.aptly_out)
            time.sleep(1)
        else:
            APITest.aptly_out.clear()

        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir, "upload")):
            shutil.rmtree(os.path.join(os.environ["HOME"], self.aptlyDir, "upload"))

        self.prepare_fixture()

    def debug_output(self):
        return APITest.aptly_out.get_contents()

    def run(self):
        pass

    def get(self, uri, *args, **kwargs):
        return requests.get("http://%s%s" % (self.base_url, uri), *args, **kwargs)

    def post(self, uri, *args, **kwargs):
        if "json" in kwargs:
            kwargs["data"] = json.dumps(kwargs.pop("json"))
            if "headers" not in kwargs:
                kwargs["headers"] = {}
            kwargs["headers"]["Content-Type"] = "application/json"
        return requests.post("http://%s%s" % (self.base_url, uri), *args, **kwargs)

    def _ensure_async(self, kwargs):
        # Make sure we run this as an async task
        params = kwargs.get('params', {})
        params.setdefault('_async', True)
        kwargs['params'] = params

    def post_task(self, uri, *args, **kwargs):
        self._ensure_async(kwargs)
        resp = self.post(uri, *args, **kwargs)
        if resp.status_code != 202:
            raise Exception("async api error: " + resp.text)

        _id = resp.json()['ID']
        resp = self.get("/api/tasks/" + str(_id) + "/wait")
        self.check_equal(resp.status_code, 200)

        return self.get("/api/tasks/" + str(_id))

    def check_task(self, task):
        self.check_equal(task.status_code, 200)
        if task.json()['State'] != TASK_SUCCEEDED:
            resp2 = self.get("/api/tasks/" + str(task.json()['ID']) + "/output")
            raise Exception(f"task failed: {resp2.text}")

    def check_task_fail(self, task, expected_output=None):
        self.check_equal(task.status_code, 200)
        if task.json()['State'] == TASK_SUCCEEDED:
            raise Exception("task expected to fail")
        if expected_output:
            resp = self.get("/api/tasks/" + str(task.json()['ID']) + "/output")
            self.check_equal(resp.json(), expected_output)

    def put(self, uri, *args, **kwargs):
        if "json" in kwargs:
            kwargs["data"] = json.dumps(kwargs.pop("json"))
            if "headers" not in kwargs:
                kwargs["headers"] = {}
            kwargs["headers"]["Content-Type"] = "application/json"
        return requests.put("http://%s%s" % (self.base_url, uri), *args, **kwargs)

    def put_task(self, uri, *args, **kwargs):
        self._ensure_async(kwargs)
        resp = self.put(uri, *args, **kwargs)
        if resp.status_code != 202:
            return resp

        _id = resp.json()['ID']
        resp = self.get("/api/tasks/" + str(_id) + "/wait")
        self.check_equal(resp.status_code, 200)

        return self.get("/api/tasks/" + str(_id))

    def delete(self, uri, *args, **kwargs):
        if "json" in kwargs:
            kwargs["data"] = json.dumps(kwargs.pop("json"))
            if "headers" not in kwargs:
                kwargs["headers"] = {}
            kwargs["headers"]["Content-Type"] = "application/json"
        return requests.delete("http://%s%s" % (self.base_url, uri), *args, **kwargs)

    def delete_task(self, uri, *args, **kwargs):
        self._ensure_async(kwargs)
        resp = self.delete(uri, *args, **kwargs)
        if resp.status_code != 202:
            return resp

        _id = resp.json()['ID']
        resp = self.get("/api/tasks/" + str(_id) + "/wait")
        self.check_equal(resp.status_code, 200)

        return self.get("/api/tasks/" + str(_id))

    def upload(self, uri, *filenames, **kwargs):
        upload_name = kwargs.pop("upload_name", None)
        directory = kwargs.pop("directory", "files")
        assert kwargs == {}

        files = {}

        for filename in filenames:
            fp = open(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), directory, filename), "rb")
            if upload_name is not None:
                upload_filename = upload_name
            else:
                upload_filename = filename
            files[upload_filename] = (upload_filename, fp)

        return self.post(uri, files=files)

    @classmethod
    def shutdown_class(cls):
        if cls.aptly_server is not None:
            cls.aptly_server.terminate()
            cls.aptly_server.wait()
            cls.aptly_server = None
        if APITest.aptly_out is not None:
            APITest.aptly_out.close()

    def random_name(self):
        return ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(15))



"""
Test aptly version
"""

from lib import BaseTest


class VersionTest(BaseTest):
    """
    version should match
    """
    gold_processor = BaseTest.expand_environ

    runCmd = "aptly version"



import requests_unixsocket
import time
import os
import urllib.error
import urllib.parse
import urllib.request

from lib import BaseTest
from testout import TestOut


class UnixSocketAPITest(BaseTest):
    aptly_server = None
    socket_path = "/tmp/_aptly_test.sock"
    base_url = ("unix://%s" % socket_path)
    aptly_out = None
    debugOutput = True

    def prepare(self):
        if self.aptly_server is None:
            UnixSocketAPITest.aptly_out = TestOut()
            self.aptly_server = self._start_process("aptly api serve -no-lock -listen=%s" % (self.base_url), stdout=UnixSocketAPITest.aptly_out, stderr=UnixSocketAPITest.aptly_out)
            time.sleep(1)
        else:
            UnixSocketAPITest.aptly_out.clear()
        super(UnixSocketAPITest, self).prepare()

    def debug_output(self):
        return UnixSocketAPITest.aptly_out.get_contents()

    def shutdown(self):
        if self.aptly_server is not None:
            self.aptly_server.terminate()
            self.aptly_server.wait()
            self.aptly_server = None
        super(UnixSocketAPITest, self).shutdown()

    def run(self):
        pass

    """
    Verify we can listen on a unix domain socket.
    """
    def check(self):
        session = requests_unixsocket.Session()
        r = session.get('http+unix://%s/api/version' % urllib.parse.quote(UnixSocketAPITest.socket_path, safe=''))
        # Just needs to come back, we actually don't care much about the code.
        # Only needs to verify that the socket is actually responding.
        self.check_equal(r.json(), {'Version': os.environ['APTLY_VERSION']})



import re

from lib import BaseTest


class EditMirror1Test(BaseTest):
    """
    edit mirror: enable filter & download sources
    """
    fixtureDB = True
    runCmd = "aptly mirror edit -filter=nginx -filter-with-deps -with-sources wheezy-main"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly mirror show wheezy-main", "mirror_show", match_prepare=lambda s: re.sub(r"Last update: [0-9:+A-Za-z -]+\n", "", s))


class EditMirror2Test(BaseTest):
    """
    edit mirror: missing mirror
    """
    runCmd = "aptly mirror edit wheezy-main"
    expectedCode = 1


class EditMirror3Test(BaseTest):
    """
    edit mirror: no changes
    """
    fixtureDB = True
    runCmd = "aptly mirror edit wheezy-main"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly mirror show wheezy-main", "mirror_show", match_prepare=lambda s: re.sub(r"Last update: [0-9:+A-Za-z -]+\n", "", s))


class EditMirror4Test(BaseTest):
    """
    edit mirror: wrong query
    """
    fixtureDB = True
    runCmd = "aptly mirror edit -filter=| wheezy-main"
    expectedCode = 1


class EditMirror5Test(BaseTest):
    """
    edit mirror: remove filter
    """
    fixtureCmds = [
        "aptly mirror create -ignore-signatures -filter='nginx | Priority (required)' mirror5 http://repo.aptly.info/system-tests/archive.debian.org/debian-security/ stretch/updates main",
    ]
    runCmd = "aptly mirror edit -filter= mirror5"

    def check(self):
        def removeDates(s):
            return re.sub(r"(Date|Valid-Until): [,0-9:+A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly mirror show mirror5", "mirror_show", match_prepare=removeDates)


class EditMirror6Test(BaseTest):
    """
    edit mirror: change architectures
    """
    fixtureCmds = [
        "aptly mirror create -ignore-signatures -architectures=amd64 mirror6 http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main"
    ]
    runCmd = "aptly mirror edit -ignore-signatures -architectures=amd64,i386 mirror6"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly mirror show mirror6", "mirror_show", match_prepare=lambda s: re.sub(r"Last update: [0-9:+A-Za-z -]+\n", "", s))


class EditMirror7Test(BaseTest):
    """
    edit mirror: change architectures to missing archs
    """
    fixtureCmds = [
        "aptly mirror create -ignore-signatures -architectures=amd64 stretch http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main"
    ]
    runCmd = "aptly mirror edit -ignore-signatures -architectures=amd64,x56 stretch"
    expectedCode = 1


class EditMirror8Test(BaseTest):
    """
    edit mirror: enable udebs
    """
    fixtureDB = True
    runCmd = "aptly mirror edit -with-udebs wheezy-main"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly mirror show wheezy-main", "mirror_show", match_prepare=lambda s: re.sub(r"Last update: [0-9:+A-Za-z -]+\n", "", s))


class EditMirror9Test(BaseTest):
    """
    edit mirror: flat mirror with udebs
    """
    fixtureCmds = ["aptly mirror create -keyring=aptlytest.gpg mirror9 http://repo.aptly.info/system-tests/pkg.jenkins.io/debian-stable binary/"]
    fixtureGpg = True
    runCmd = "aptly mirror edit -with-udebs mirror9"
    expectedCode = 1


class EditMirror10Test(BaseTest):
    """
    edit mirror: change archive url
    """
    fixtureCmds = ["aptly mirror create -ignore-signatures mirror10 http://repo.aptly.info/system-tests/ftp.ru.debian.org/debian bookworm main"]
    runCmd = "aptly mirror edit -ignore-signatures -archive-url http://repo.aptly.info/system-tests/ftp.ch.debian.org/debian mirror10"



from lib import BaseTest


class PublishList1Test(BaseTest):
    """
    publish list: empty list
    """
    runCmd = "aptly publish list"


class PublishList2Test(BaseTest):
    """
    publish list: several repos list
    """
    requiresGPG2 = True
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot merge snap2 snap1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
        "aptly -architectures=amd64 publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=wheezy -component=contrib snap2 ppa/smira",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -origin=origin1 snap2 ppa/tr1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -label=label1 snap2 ppa/tr2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -component=main,contrib snap1 snap2 ppa",
    ]
    runCmd = "aptly publish list"


class PublishList3Test(BaseTest):
    """
    publish list: several repos list, raw
    """
    requiresGPG2 = True
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot merge snap2 snap1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
        "aptly -architectures=amd64 publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=wheezy -component=contrib snap2 ppa/smira",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -origin=origin1 snap2 ppa/tr1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -label=label1 snap2 ppa/tr2",
    ]
    runCmd = "aptly publish list -raw"


class PublishList4Test(BaseTest):
    """
    publish list json: empty list
    """
    runCmd = "aptly publish list -json"


class PublishList5Test(BaseTest):
    """
    publish list json: several repos list
    """
    requiresGPG2 = True
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot merge snap2 snap1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
        "aptly -architectures=amd64 publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=wheezy -component=contrib snap2 ppa/smira",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -origin=origin1 snap2 ppa/tr1",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -label=label1 snap2 ppa/tr2",
    ]
    runCmd = "aptly publish list -json"



from lib import BaseTest


class PublishShow1Test(BaseTest):
    """
    publish show: existing snapshot
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
    ]
    runCmd = "aptly publish show maverick"


class PublishShow2Test(BaseTest):
    """
    publish show: under prefix
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 ppa/smira",
    ]
    runCmd = "aptly publish show maverick ppa/smira"


class PublishShow3Test(BaseTest):
    """
    publish show json: existing snapshot
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
    ]
    runCmd = "aptly publish show -json maverick"


class PublishShow4Test(BaseTest):
    """
    publish show json: under prefix
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 ppa/smira",
    ]
    runCmd = "aptly publish show -json maverick ppa/smira"


class PublishShow5Test(BaseTest):
    """
    publish show: existing local repo
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly repo create -distribution=wheezy local-repo",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -architectures=i386 local-repo"
    ]
    runCmd = "aptly publish show wheezy"
    gold_processor = BaseTest.expand_environ



from fs_endpoint_lib import FileSystemEndpointTest


class FSEndpointPublishSnapshot1Test(FileSystemEndpointTest):
    """
    publish snapshot: using symlinks
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot1Test, self).check()

        self.check_is_regular('public_symlink/dists/maverick/InRelease')
        self.check_is_regular('public_symlink/dists/maverick/Release')
        self.check_is_regular('public_symlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_symlink('public_symlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot2Test(FileSystemEndpointTest):
    """
    publish snapshot: using hardlinks
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot2Test, self).check()

        self.check_is_regular('public_hardlink/dists/maverick/InRelease')
        self.check_is_regular('public_hardlink/dists/maverick/Release')
        self.check_is_regular('public_hardlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_hardlink('public_hardlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot3Test(FileSystemEndpointTest):
    """
    publish snapshot: using copy
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot3Test, self).check()

        self.check_is_regular('public_copy/dists/maverick/InRelease')
        self.check_is_regular('public_copy/dists/maverick/Release')
        self.check_is_regular('public_copy/dists/maverick/Release.gpg')

        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_copy/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_copy/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_copy/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_copy/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_copy('public_copy/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot4Test(FileSystemEndpointTest):
    """
    publish snapshot: using copy, symlink and hardlink variants
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:",
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot4Test, self).check()

        self.check_is_regular('public_copy/dists/maverick/InRelease')
        self.check_is_regular('public_copy/dists/maverick/Release')
        self.check_is_regular('public_copy/dists/maverick/Release.gpg')

        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_copy/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_copy/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_copy/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_copy/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_copy/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_copy('public_copy/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_is_regular('public_symlink/dists/maverick/InRelease')
        self.check_is_regular('public_symlink/dists/maverick/Release')
        self.check_is_regular('public_symlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_symlink('public_symlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_is_regular('public_hardlink/dists/maverick/InRelease')
        self.check_is_regular('public_hardlink/dists/maverick/Release')
        self.check_is_regular('public_hardlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_hardlink('public_hardlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot5Test(FileSystemEndpointTest):
    """
    publish snapshot: using copy, symlink and hardlink variants under prefixes
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:snap_copy/daily",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:snap_symlink/daily",
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:snap_hardlink/daily"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot5Test, self).check()

        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/InRelease')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/Release')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/Release.gpg')

        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_copy/snap_copy/daily/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_copy/snap_copy/daily/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_copy/snap_copy/daily/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_copy('public_copy/snap_copy/daily/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/InRelease')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/Release')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/Release.gpg')

        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_symlink/snap_symlink/daily/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_symlink/snap_symlink/daily/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_symlink/snap_symlink/daily/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_symlink('public_symlink/snap_symlink/daily/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/InRelease')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/Release')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/Release.gpg')

        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_hardlink/snap_hardlink/daily/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_hardlink/snap_hardlink/daily/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_hardlink/snap_hardlink/daily/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_hardlink('public_hardlink/snap_hardlink/daily/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot6Test(FileSystemEndpointTest):
    """
    publish snapshot: drop one
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:"
    ]
    runCmd = "aptly publish drop maverick filesystem:copy:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot6Test, self).check()

        self.check_not_exists('public_copy/dists/')
        self.check_not_exists('public_copy/pool/')

        self.check_is_regular('public_symlink/dists/maverick/InRelease')
        self.check_is_regular('public_symlink/dists/maverick/Release')
        self.check_is_regular('public_symlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_symlink('public_symlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_is_regular('public_hardlink/dists/maverick/InRelease')
        self.check_is_regular('public_hardlink/dists/maverick/Release')
        self.check_is_regular('public_hardlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_hardlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_hardlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_hardlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_hardlink('public_hardlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')


class FSEndpointPublishSnapshot7Test(FileSystemEndpointTest):
    """
    publish snapshot: drop two
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:",
        "aptly publish drop maverick filesystem:copy:"
    ]
    runCmd = "aptly publish drop maverick filesystem:hardlink:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot7Test, self).check()

        self.check_not_exists('public_copy/dists/')
        self.check_not_exists('public_copy/pool/')

        self.check_is_regular('public_symlink/dists/maverick/InRelease')
        self.check_is_regular('public_symlink/dists/maverick/Release')
        self.check_is_regular('public_symlink/dists/maverick/Release.gpg')

        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-i386.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Release')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_is_regular('public_symlink/dists/maverick/main/binary-amd64/Packages.bz2')
        self.check_is_regular('public_symlink/dists/maverick/main/Contents-amd64.gz')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-i386/Packages')
        self.check_not_exists('public_symlink/dists/maverick/main/debian-installer/binary-amd64/Packages')

        self.check_is_symlink('public_symlink/pool/main/g/gnuplot/gnuplot-doc_4.6.1-1~maverick2_all.deb')

        self.check_not_exists('public_hardlink/dists/')
        self.check_not_exists('public_hardlink/pool/')


class FSEndpointPublishSnapshot8Test(FileSystemEndpointTest):
    """
    publish snapshot: remove snapshot
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:",
        "aptly publish drop maverick filesystem:copy:",
        "aptly publish drop maverick filesystem:symlink:",
        "aptly publish drop maverick filesystem:hardlink:",
    ]
    runCmd = "aptly snapshot drop snap1"
    gold_processor = FileSystemEndpointTest.expand_environ


class FSEndpointPublishSnapshot9Test(FileSystemEndpointTest):
    """
    publish snapshot: remove snapshot error
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:",
        "aptly publish drop maverick filesystem:copy:",
    ]
    runCmd = "aptly snapshot drop snap1"
    expectedCode = 1


class FSEndpointPublishSnapshot10Test(FileSystemEndpointTest):
    """
    publish list: several repos list
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:copy:",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:symlink:snap_symlink/daily",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1 filesystem:hardlink:"
    ]
    runCmd = "aptly publish list -raw"


class FSEndpointPublishSnapshot11Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using symlink method
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:symlink:"
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:symlink:"
    expectedCode = 1
    gold_processor = FileSystemEndpointTest.expand_environ


class FSEndpointPublishSnapshot12Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using symlink method. -force-overwrite
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:symlink:"
    ]
    runCmd = "aptly publish snapshot -force-overwrite -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:symlink:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot12Test, self).check()

        self.check_file_contents("public_symlink/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz", "file")


class FSEndpointPublishSnapshot13Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with md5 verification
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copy:"
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copy:"
    expectedCode = 1
    gold_processor = FileSystemEndpointTest.expand_environ


class FSEndpointPublishSnapshot14Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with md5 verification. -force-overwrite
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copy:"
    ]
    runCmd = "aptly publish snapshot -force-overwrite -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copy:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot14Test, self).check()

        self.check_file_contents("public_copy/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz", "file")


class FSEndpointPublishSnapshot15Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with size verification
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copysize:"
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copysize:"
    expectedCode = 1
    gold_processor = FileSystemEndpointTest.expand_environ


class FSEndpointPublishSnapshot16Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with size verification. -force-overwrite
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${files}",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copysize:"
    ]
    runCmd = "aptly publish snapshot -force-overwrite -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copysize:"
    gold_processor = FileSystemEndpointTest.expand_environ

    def check(self):
        super(FSEndpointPublishSnapshot16Test, self).check()

        self.check_file_contents("public_copysize/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz", "file")


class FSEndpointPublishSnapshot17Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with md5 verification
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${testfiles}/1",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}/2",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copy:"
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copy:"
    expectedCode = 1
    gold_processor = FileSystemEndpointTest.expand_environ


class FSEndpointPublishSnapshot18Test(FileSystemEndpointTest):
    """
    publish snapshot: conflicting files in the snapshot using copy method with size verification (not detected!)
    """
    fixtureCmds = [
        "aptly repo create local-repo1",
        "aptly repo add local-repo1 ${testfiles}/1",
        "aptly snapshot create snap1 from repo local-repo1",
        "aptly repo create local-repo2",
        "aptly repo add local-repo2 ${testfiles}/2",
        "aptly snapshot create snap2 from repo local-repo2",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 filesystem:copysize:"
    ]
    runCmd = "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=squeeze snap2 filesystem:copysize:"
    gold_processor = FileSystemEndpointTest.expand_environ



import os
from lib import BaseTest


class FileSystemEndpointTest(BaseTest):
    """
    BaseTest + support for filesystem endpoints
    """

    def prepare(self):
        self.configOverride = {"FileSystemPublishEndpoints": {
            "symlink": {
                "rootDir": os.path.join(os.environ["HOME"], ".aptly", "public_symlink"),
                "linkMethod": "symlink"
            },
            "hardlink": {
                "rootDir": os.path.join(os.environ["HOME"], ".aptly", "public_hardlink"),
                "linkMethod": "hardlink"
            },
            "copy": {
                "rootDir": os.path.join(os.environ["HOME"], ".aptly", "public_copy"),
                "linkMethod": "copy",
                "verifyMethod": "md5"
            },
            "copysize": {
                "rootDir": os.path.join(os.environ["HOME"], ".aptly", "public_copysize"),
                "linkMethod": "copy",
                "verifyMethod": "size"
            }
        }}
        super(FileSystemEndpointTest, self).prepare()

    def check_is_regular(self, path):
        if not os.path.isfile(os.path.join(os.environ["HOME"], ".aptly", path)):
            raise Exception("path %s is not a regular file" % (path, ))

    def check_is_symlink(self, path):
        if not os.path.islink(os.path.join(os.environ["HOME"], ".aptly", path)):
            raise Exception("path %s is not a symlink" % (path, ))

    def is_hardlink(self, path):
        return os.stat(os.path.join(os.environ["HOME"], ".aptly", path)).st_nlink >= 2

    def check_is_hardlink(self, path):
        if not self.is_hardlink(path):
            raise Exception("path %s is not a hardlink" % (path, ))

    def check_is_copy(self, path):
        fullpath = os.path.join(os.environ["HOME"], ".aptly", path)
        if not (os.path.isfile(fullpath) and not self.is_hardlink(path)):
            raise Exception("path %s is not a copy" % (path, ))
